
const String sPrgm="LazyNamedClock-18c14";
const String sVers="17 Dec 2018";

/*====
Creates a simple pulse counter.

There are fancier, better ways to do that!

But this WORKS. May be all you need. And along the way
demonstrates other stuff.

See...


... for details.

*/
//===============================================
//Pin assignments....

//The following is used by LazyClock.
//In LazyClock, this pin is renamed pPendInFrontOfLED... use that
//name throughout LazyClock... but if you want that on a
//different pin, change this. (pLED0 is used to keep
//this project NoviceGuard/ NoviceGuardLite friendly.)
const byte pLED0=4;//An LED, tied low through a suitable resistor
//is on this line, can be used for output to user.

//Following... not(? yet?) used by LazyClock
const byte pLED1=5;//An LED, tied low through a suitable resistor
//is on this line, can be used for output to user.

//Following... not(? yet?) used by LazyClock
const byte pLED2=12;//An LED, tied low through a suitable resistor
//is on this line, can be used for output to user.

//Following... not(? yet?) used by LazyClock
const byte pLED3=13;//An LED, tied low through a suitable resistor
//is on this line, can be used for output to user.
//On many Arduinos, there is, of course, an LED on the Arduino
//itself attached (with resistor) to this pin.

//Following... not(? yet?) used by LazyClock
const byte pBu0=A3;//A momentary switch/button, SPST, NO is on this,
//ANALOG IO line A3. If the button is pressed, the line is pulled low.
//The use of an analog line may seem odd. It is used to keep this
//hardware compatible with my NoviceGuard and NoviceGuardLite
//Arduino hardware environments.

//Following... not(? yet?) used by LazyClock
const byte bBu1=2;//A momentary switch/button, SPST, NO is on this,
//IO line D2. If the button is pressed, the line is pulled low.
//It is used to keep this hardware compatible 
//with my NoviceGuard and NoviceGuardLite
//Arduino hardware environments.

//Following used by LazyClock for input of "a swing" pulse from
//monitored pendulum. (As discussed elsewhere, the system can be
//set up to generate two pulses per swing.)
//In LazyClock, this pin is renamed pPendulumSensor... use that
//name thoughout LazyClock. (pBu2 is used to keep
//this project NoviceGuard/ NoviceGuardLite friendly.)
const byte pBu2=3;//A momentary switch/button, SPST, NO is on this,
//IO line D2. If the button is pressed, the line is pulled low.
//It is used to keep this hardware compatible 
//with my NoviceGuard and NoviceGuardLite
//Arduino hardware environments.


//"PECULIAR" TO LazyClock....

//Global variables, etc

//Next two used for serial connection to external display
//Only pSevenSegLEDSerialFROMArdu actually USED by
//this program, but the library used to create the port
//doesn't provide for a case where the Arduino is only
//sending, so a second line has to be set aside. It will
//be configured as an input. Probably best to leave it
//unused, and perhaps even tied high or low, although
//I don't think I've ever bothered to do that.
const byte pSevenSegLEDSerialTOArdu=10;
const byte pSevenSegLEDSerialFROMArdu=11;

#include <SoftwareSerial.h>
// "NewSoftSerial.h", the old way, is not available
// in standard Arduino 1.8.5. SoftwareSerial works.

SoftwareSerial SevenSegLED(pSevenSegLEDSerialTOArdu,
   pSevenSegLEDSerialFROMArdu);//For Sparkfun  COM-11442

byte pPendulumSensor=9;//Assign a pin to be the one
//watched for an input to the app that says "Pendulum
//in front of sensor. Invert, or not, the signal
//INSIDE boPendInFrontOfSensor(), so that the app
//is isolated from changes in hardware. (Some sensors
//will return high when sensor in front, others low.)
//This is the "real" sensor...
//see also pPendulumSensorButton


byte pPendulumSensorButton=pBu2;//Re-name a NoviceGuard
//pin with something more meaningful in LazyClock context.
//This pin will be configured as an input. The input will
//be connected to a button which, when pressed, will "count
//as" the pendulum being in front of the sensor. It is
//for debugging work only.
//If pulse stretching is implemented the "in front of" the
//sensor state should persist a bit after the button is
//released.

//----------
//Following drives the "state machine" nature of the program
//When it comes true, the program does most of what it must
//do for each pulse from the pendulim, and then goes over to
//watching boPendInFrontOfSensor(). As soon as it
//sees, from that, the pendulum in front of the sensor, it
//goes into a blocked loop, waiting for the pendulum to be
//PAST the sensor (and for any stretching, if present, to
//be over.)

boolean boWaitingForDetected;

const byte pPendInFrontOfLED=pLED0;//Re-name a NoviceGuard pin with
//something more meaningful in LazyClock context. This
//pin will be configured as an output; it is connected to LED tied
//through a resistor to ground. In LazClock, if on, the pendulum is
//currently "seen" as being in front of the sensor... although it may
//be on a little longer than that is actually the case, if external
//pulse stretching has been implemented.

//--------  ulPulsesFrmPendulum  -------------------------
//The count of pulses seen.
//Unsigned long is overkill at the moment... though not
//entirely irrelevant.
//The currently provided display is 4 decimal digits.
//However, for the leap seconds, etc, using unsigned long
//means that artifacts arising due to roll-over are minimized.
unsigned long ulPulsesFrmPendulum;

//Forward declarations for things in other tabs of Arduino IDE...
boolean boPendInFrontOfSensor();
void LEDOn(byte bWhich,boolean boOn);

void setup() {
Serial.begin(9600);
delay(50);
Serial.println("Hi from "+sPrgm);
Serial.println("   vers "+sVers);

SevenSegLED.begin(9600);
delay(50);
SevenSegLED.print("v");//To reset display module
SevenSegLED.print("8888");

pinMode(pPendulumSensor,INPUT);//This fed, 16 Dec 2018, bt an
//output from a separate 5v Arduino, so a good "0" or
//"5"v delivered, Pull-up not needed. High when pendulum
//in front of sensor
pinMode(pPendulumSensorButton,INPUT_PULLUP);
pinMode(pPendInFrontOfLED,OUTPUT);

ulPulsesFrmPendulum=0;
boWaitingForDetected=true;

}//end of setup()

void loop() {
int iToDisplay;

if (boWaitingForDetected)//============ then ===== 1
{
  if (boPendInFrontOfSensor())//======= then ===== 2
  //now in HAS BEEN detected. Stay here 'til NOT
  //  detected
  {
    LEDOn(pPendInFrontOfLED,true);
    do//============================= do/while === 3
    {
      delay(5);     
      } while (boPendInFrontOfSensor());
      boWaitingForDetected=false;
      //end of do... while loop------------------- 3
  }//no ; here//----- end of then/ else starts === 2
  else//Pend no longer in front of sensor
  {  
  }//end of else of if (boPendInFrontOfSensor())
}//no ; here//------- end of then/ else starts === 1
else //do the things that should be done once,
//going into has been detected, which then sets us
//back to waiting for detected....
{
  LEDOn(pPendInFrontOfLED,false);
  ulPulsesFrmPendulum++;
  iToDisplay=ulPulsesFrmPendulum % 10000;
  SevenSegLED.print("v");//To reset display module
  delay(50);
  SevenSegLED.print(iToDisplay);
  //Jobs done... now go back to...
  boWaitingForDetected=true;
}//end of else of if (boWaitingForDetected)

}//end of loop()
